<?php
namespace Controller;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

class Solicitacao extends \Zeedhi\Framework\Controller\Simple {

	protected $solicitacao;

	public function __construct(\Service\Solicitacao $solicitacao) {
        $this->solicitacao = $solicitacao;
    }

	public function retornaDatasEntregaSolicitacao(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $datasEntraga = $this->solicitacao->retornaDatasEntregaSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('DatasEntregaSolicitacao', $datasEntraga));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProdutosSolicitacao(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $produtos = $this->solicitacao->retornaProdutosSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('ProdutosSolicitacao', $produtos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaSolicitacoes(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $solicitacoes = $this->solicitacao->retornaSolicitacoes($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('Solicitacao', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProximoCodigoSolicitacao(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];

            $codigo = $this->solicitacao->retornaProximoCodigoSolicitacao($nrorg, $cdfilial);

			$response->addDataSet(new DataSet('ProximoCodigoSolicitacao', $codigo));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function novaSolicitacao(Request\Dataset $request, Response $response) {
        
        try {

            $rows = $request->getDataSet()->getRows();

            $this->solicitacao->novaSolicitacao($rows);

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));

        }
    }

    public function salvaProdutoEditado(Request\Dataset $request, Response $response) {
       
        try {

            $rows = $request->getDataSet()->getRows();

            $produtosJaAprovados = $this->solicitacao->salvaProdutoEditado($rows);

            $response->addDataSet(new Dataset('ProdutosAprovados', $produtosJaAprovados));

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function excluirProdutos(Request\Dataset $request, Response $response) {
       
        try {

            $rows = $request->getDataSet()->getRows();
            $produtosJaAprovados = array();

            foreach ($rows as $row) {

                $produtoJaAprovado = $this->solicitacao->excluirProdutos($row);
                if (!empty($produtoJaAprovado)) {
                    $produtosJaAprovados[] = $produtoJaAprovado;
                }

            }

            $response->addDataSet(new Dataset('ProdutosAprovados', $produtosJaAprovados));

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));

        }
    }
}