<?php
/**
 * Created by Luiz Filho
 */

namespace Model\Repository;

use Doctrine\ORM\EntityRepository;

class Solicitacao extends EntityRepository {

    public function retornaDatasEntregaSolicitacao($nrorg, $filiais, $dataInicial, $dataFinal) {
        $dataSet = array();
	  	$sql = "SELECT
            DISTINCT TO_CHAR(ITEMSOLI.DTPROGENTR, 'DD/MM/YYYY') AS DATA,
            TO_CHAR(ITEMSOLI.DTPROGENTR, 'DD/MM/YYYY - DAY', 'NLS_DATE_LANGUAGE= ''BRAZILIAN PORTUGUESE''') AS DATA_EXTENSO,
            ITEMSOLI.CDFILSOLI
            FROM ITEMSOLI
            INNER JOIN PRODUTO ON PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO
            INNER JOIN (SELECT NVL(IDEXPSOLCMP, 'N') IDEXPSOLCMP, F.CDFILIAL
            FROM FILIAL F 
                LEFT JOIN PARASUPRI P ON (NVL(F.CDFILCENTCMP, F.CDFILIAL) = P.CDFILIAL)
                WHERE F.CDFILIAL = :CDFILIAL
                AND F.NRORG = :NRORG) PARAM ON PARAM.CDFILIAL = ITEMSOLI.CDFILSOLI
            WHERE ITEMSOLI.DTPROGENTR BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            AND ITEMSOLI.CDFILSOLI = :CDFILIAL
            AND ITEMSOLI.NRORG = :NRORG
            AND (ITEMSOLI.IDSOLIAPROV = PARAM.IDEXPSOLCMP OR 'S' = PARAM.IDEXPSOLCMP)
            ORDER BY TO_DATE(DATA, 'DD/MM/YYYY') ASC";

	  	$params = array(
		    'NRORG' => $nrorg,
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'CDFILIAL' => ''
		);

        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaProdutosSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal) {

        $parametro = $this->parametroSolicitacaoAprovada($cdfilial, $nrorg);

        $sql = "SELECT ROWNUM, itemsoli.cdfillan||' - '||itemsoli.nrsolicmp AS CDFILICONCATNRSOL, itemsoli.cdfillan, itemsoli.cdfilsoli, itemsoli.nrsolicmp, produto.cdproduto, 
                nmproduto, sgunidade, qtsoliprod, TO_CHAR(dtprogentr,'DD/MM/YYYY') AS dtprogentr,
                cdfiliopelog, idsoliaprov, itemsoli.nrorg, NULL AS qtajustada, NULL AS dsjustoper,
                itemsoli.iditmsolextra, movsolext.DSJUSTSOLEXT
                FROM itemsoli
                INNER JOIN solicita ON  itemsoli.cdfillan = solicita.CDFILLANCA AND itemsoli.cdfilsoli = solicita.cdfilsoli AND itemsoli.NRSOLICMP = solicita.NRSOLICMP
                INNER JOIN produto ON produto.cdproduto = itemsoli.cdproduto AND produto.nrorg = itemsoli.nrorg
                LEFT JOIN movsolext ON movsolext.cdfillancext = itemsoli.cdfillan AND movsolext.cdfilsolext = itemsoli.cdfilsoli AND movsolext.nrsolimovext = itemsoli.nrsolicmp AND TRUNC(movsolext.DTMOVSOLEXT) = TRUNC(solicita.DTSOLCMP)
                WHERE ITEMSOLI.DTPROGENTR BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
                AND itemsoli.cdfilsoli =:cdfilial 
                AND itemsoli.nrorg =:nrorg
                AND (itemsoli.idsoliaprov =:idsoliaprov OR 'S' =:idsoliaprov)
                ORDER BY nmproduto, TO_DATE(dtprogentr,'DD/MM/YYYY')";
        $params = array(
            'NRORG' => $nrorg,
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'IDSOLIAPROV' => $parametro,
            'CDFILIAL' => $cdfilial
        );

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return $result;
    }

    private function parametroSolicitacaoAprovada($cdfilial, $nrorg) {

        $sql = "SELECT NVL(IDEXPSOLCMP, 'N') IDEXPSOLCMP
            FROM FILIAL F 
                LEFT JOIN PARASUPRI P ON (NVL(F.CDFILCENTCMP, F.CDFILIAL) = P.CDFILIAL)
                WHERE F.CDFILIAL = :CDFILIAL
                AND F.NRORG = :NRORG";

        $params = array(
            'NRORG' => $nrorg,
            'CDFILIAL' => $cdfilial,
        );

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return isset($result[0]) ? $result[0]['IDEXPSOLCMP'] : 'N';
    }

    public function retornaSolicitacoes($nrorg, $filiais, $dataInicial, $dataFinal) {
        $dataSet = array();
        $sql = "SELECT CDFILLANCA,
              CDFILSOLI,
              NRSOLICMP,
              TO_CHAR(DTSOLCMP, 'DD/MM/YYYY') AS DTSOLCMP
            FROM SOLICITA
            WHERE CDFILSOLI = :CDFILIAL
            AND DTSOLCMP BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            AND NRORG = :NRORG
            ORDER BY DTSOLCMP DESC,
              NRSOLICMP DESC";

        $params = array(
            'NRORG' => $nrorg,
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'CDFILIAL' => '',
        );
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaProximoCodigoSolicitacao($nrorg, $filiais) {
        $dataSet = array();
        $sql = "SELECT
              LPAD((NVL(MAX(NRSOLICMP),0) + 1),10,'0') PROXIMOCODIGO,
              CDFILSOLI
            FROM SOLICITA
            WHERE CDFILSOLI = :CDFILIAL
            AND NRORG = :NRORG
            GROUP BY CDFILSOLI";

        $params = array(
            'NRORG' => $nrorg,
            'CDFILIAL' => '',
        );
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaNumeroSolicitacao($row) {

        $sql = "SELECT
              LPAD((NVL(MAX(NRSOLICMP),0) + 1),10,'0') PROXIMOCODIGO,
              CDFILSOLI
            FROM SOLICITA
            WHERE CDFILSOLI = :CDFILSOLI
            AND CDFILLANCA = :CDFILLANCA
            AND NRORG = :NRORG
            GROUP BY CDFILSOLI";

        $params = array(
            'NRORG' => $row['NRORG'],
            'CDFILSOLI' => $row['CDFILSOLI'],
            'CDFILLANCA' => $row['CDFILLANCA'],
        );

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        if(empty($result)) {
            $result = array(
                array(
                    'PROXIMOCODIGO' => '0000000001',
                    'CDFILSOLI' => $row['CDFILSOLI'],
                )
            );
        }

        return $result[0]['PROXIMOCODIGO'];
        
    }

    public function retornaCdfiliopelog($produto, $cdfilcentcmp) {
        if ($this->consolidaSoliCompParaOL($cdfilcentcmp)) {

            $sql = "SELECT MAX(CDFILIOPELOG) AS CDFILIOPELOG
                FROM PARAMPRODSUP
                WHERE CDFILIAL = :CDFILIAL
                AND CDPRODUTO = :CDPRODUTO
                AND NRORG = :NRORG";

            $params = array(
                'NRORG' => $produto['NRORG'],
                'CDPRODUTO' => $produto['CDPRODUTO'],
                'CDFILIAL' => $produto['CDFILSOLI'],
            );

            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();

            if (empty($result)) {
                return null;
            } else {
                return $result[0]['CDFILIOPELOG'];
            }
        } else {
            return null;
        }

    }

    private function consolidaSoliCompParaOL($cdfilial) {
        $sql = "SELECT IDUTILCONSSOLIOL FROM PARASUPRI WHERE CDFILIAL = :CDFILIAL";
        $params = array('CDFILIAL' => $cdfilial);

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        if (!empty($result)) {
            if ($result[0]['IDUTILCONSSOLIOL'] == 'S') {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
}